@@ -1,6 +1,12 @@
 # based on: Perez, J., Mendez, F. J., Menendez, M., & Losada, I. J. (2014).
 # ESTELA: a method for evaluating the source and travel time of the wave energy reaching a local area.
 # Ocean Dynamics, 64(8), 1181–1191. https://doi.org/10.1007/s10236-014-0740-7
+#
+# Examples (from olas.estela import calc, plot):
+# calc with constant spread: estelas = calc("/data_local/tmp/glob2018??01T00.nc", 46, -131, mask="MAPSTA")
+# calc with spread: estelas = calc("/data_local/tmp/ww3.glob_24m.2010??.nc", 46, -131, "hs.", "tp.", "th.", "si.", "MAPSTA")
+# plot: plot(estelas, outdir=".")
+
 import argparse
 import datetime
 import os
@@ -13,25 +19,22 @@ from cartopy import crs as ccrs
 from cartopy.io import shapereader as shpreader
 from dask.diagnostics import ProgressBar
 from matplotlib import pyplot as plt
+from scipy import special
 
 d2r = np.pi / 180.0
-gamma_fun = (
-    lambda x: np.sqrt(2.0 * np.pi / x)
-    * ((x / np.exp(1.0)) * np.sqrt(x * np.sinh(1.0 / x))) ** x
-)  # alternative to scipy.special.gamma
 
 
 def parser():
     parser = argparse.ArgumentParser(description="Calculate estelas")
     parser.add_argument("datafiles", type=str, help="Files with wave data")
-    parser.add_argument("lat0", type=float, help="Latitude target point")
-    parser.add_argument("lon0", type=float, help="Longitude target point")
-    parser.add_argument("--hs", type=str, default="hs", help="Significant wave height")
-    parser.add_argument("--tp", type=str, default="tp", help="Peak period")
-    parser.add_argument("--dp", type=str, default="dp", help="Wave direction")
-    parser.add_argument("--si", default=20, help="Directional spread")
-    parser.add_argument("-m", "--mask", type=str, default=None, help="mask")
-    parser.add_argument("-g", "--groupers", nargs="*", default=None, help="groupers")
+    parser.add_argument("lat0", type=float, help="Latitude of the target point")
+    parser.add_argument("lon0", type=float, help="Longitude of the target point")
+    parser.add_argument("--hs", type=str, default="hs", help="Significant wave height fieldnames")
+    parser.add_argument("--tp", type=str, default="tp", help="Peak period fieldnames")
+    parser.add_argument("--dp", type=str, default="dp", help="Wave direction fieldnames")
+    parser.add_argument("--si", default=20, help="Directional spread fieldnames")
+    parser.add_argument("-m", "--mask", type=str, default=None, help="mask fieldname")
+    parser.add_argument("-g", "--groupers", nargs="*", default=None, help="groupers for results")
     parser.add_argument("-p", "--proj", type=str, default=None, help="projection")
     parser.add_argument("-o", "--outdir", type=str, default=None, help="output directory")
     args = parser.parse_args()
@@ -48,10 +51,10 @@ def calc(datafiles, lat0, lon0, hs="phs.", tp="ptp.", dp="pdir.", si=20, mask=No
         datafiles (str/list): Regular expression or list of data files.
         lat0 (float): Latitude of target point.
         lon0 (float): Longitude of target point.
-        hs (str): Information of hs field names in datafiles
-        tp (str): Information of tp field names in datafiles
-        dp (str): Information of dp field names in datafiles
-        si (str/float): Information of directional spread
+        hs (str/list): regex/list of hs field names in datafiles
+        tp (str/list): regex/list of tp field names in datafiles
+        dp (str/list): regex/list of dp field names in datafiles
+        si (str/list/float): Value or regex/list of directional spread field names
         mask (str): Information of mask
         groupers (list, optional): values used to group the results.
 
@@ -93,7 +96,7 @@ def calc(datafiles, lat0, lon0, hs="phs.", tp="ptp.", dp="pdir.", si=20, mask=No
     th1_cos = np.cos(0.5 * bearings * d2r)
 
     if isinstance(mask, str):
-        mask = dsf[spec_info[mask]]
+        mask = dsf[mask]
     vland = geographic_mask(lat0, lon0, dists, bearings, mask=mask)
 
     # S and Stp calculations
@@ -113,19 +116,19 @@ def calc(datafiles, lat0, lon0, hs="phs.", tp="ptp.", dp="pdir.", si=20, mask=No
 
             if si_calculations:
                 if num_si:  # don't repeat calculations
-                    si_calculations = False
                     si = spec_info["si"]
+                    si_calculations = False
                 else:
                     si = dsf[spec_info["si"][ipart]].clip(15., 45.)
                     # TODO find better solution to avoid invalid A2 values
 
                 s = (2 / (si * np.pi / 180) ** 2) - 1
-                A2 = gamma_fun(s + 1) / (gamma_fun(s + 0.5) * 2 * np.pi ** 0.5)
+                A2 = special.gamma(s + 1) / (special.gamma(s + 0.5) * 2 * np.pi ** 0.5)
                 # TODO find faster spread approach (use normal distribution or table?)
                 coef_spread = A2 * np.pi / 180  # deg
                 # TODO review coef_spread units and compare with wavespectra
 
-            th2 = 0.5 * np.deg2rad(dp)
+            th2 = 0.5 * dp * d2r
             coef_direction = abs(np.cos(th2) * th1_cos + np.sin(th2) * th1_sin) ** (
                 2.0 * s
             )
@@ -179,76 +182,74 @@ def plot(estelas, groupers=None, proj=None, cmap="plasma", figsize=[25, 10], out
     Returns:
         figs: list of figure handles
     """
+    lon0 = float(estelas.lon0)
+    lat0 = float(estelas.lat0)
     if proj is None:
-        proj = ccrs.PlateCarree(central_longitude=float(estelas.lon0))
+        # proj = ccrs.Orthographic(lon0, lat0)
+        proj = ccrs.PlateCarree(central_longitude=lon0)
+
+    c1day = dict(levels=np.linspace(1, 30, 30), colors="grey", linewidths=0.5)
+    c3day = dict(levels=np.linspace(3, 30, 10), colors="black", linewidths=1.0)
 
     figs = []
     groupers = get_groupers(groupers)
-    for grouper in groupers:  # TODO add empty plots when no data for some of the subplots
-        if grouper == "ALL":
-            ds = estelas.sel(time="ALL")
-        elif grouper == "time.season":
-            ds = estelas.sel(time=["DJF", "MAM", "JJA", "SON"])
+    for grouper in groupers:
+        if grouper == "time.season":
+            time = ["DJF", "MAM", "JJA", "SON"]
         elif grouper == "time.month":
-            ds = estelas.sel(time=[f"m{m:02g}" for m in range(1, 13)])
+            time = [f"m{m:02g}" for m in range(1, 13)]
         else:
-            ds = estelas.copy()
+            time = [grouper]
 
-        print(f"Plotting estela for {ds}\n")  # TODO refactor plotting
-        if ds.time.size == 1:
+        timesize = len(time)
+        ds = estelas.sel(time=[t for t in time if t in estelas.time])
+        Faux = [ds.isel(time=0).assign(time=t)["F"]*np.nan for t in time if t not in estelas.time]
+        F = xr.concat([ds.F] + Faux, dim="time").sel(time=time)
+
+        print(f"Plotting estelas for time={time} from {ds}\n")  # TODO refactor plotting
+        if timesize == 1:
             fig = plt.figure(figsize=figsize)
-            ds.F.plot(
+            F.plot(
                 subplot_kws={"projection": proj},
                 transform=ccrs.PlateCarree(),
                 cmap=cmap,
             )
         else:
-            g = ds.F.plot(
+            g = F.plot(
                 subplot_kws={"projection": proj},
                 transform=ccrs.PlateCarree(),
                 cmap=cmap,
                 col="time",
-                col_wrap=2 if ds.time.size <= 4 else 3,
+                col_wrap=2 if timesize <= 4 else 3,
             )
             fig = g.fig
             fig.set_figwidth(figsize[0])
             fig.set_figheight(figsize[1])
 
         axes = fig.axes[:-1]
-        for i, ax in enumerate(axes):
-            if len(axes) > 1:
-                ttime = ds.traveltime.isel(time=i)
-            else:
-                ttime = ds.traveltime
+        for iax, ax in enumerate(axes):
+            if time[iax] not in estelas.time:
+                continue
 
-            ttime.plot.contour(
-                ax=ax,
-                transform=ccrs.PlateCarree(),
-                colors="grey",
-                linestyles="dashed",
-                levels=np.linspace(1, 30, 30),
-                add_labels=False,
-                linewidths=0.5,
-                zorder=2,
-            )
-
-            p = ttime.plot.contour(
-                ax=ax,
-                transform=ccrs.PlateCarree(),
-                colors="black",
-                linestyles="dashed",
-                levels=np.linspace(3, 30, 10),
-                add_labels=False,
-                linewidths=2,
-                zorder=4,
-            )
-            ax.clabel(p, np.linspace(3, 30, 10), colors="black", fmt="%.0fdays")
+            ttime = ds.traveltime.sel(time=time[iax])
+            for ic, c_args in enumerate([c1day, c3day]):
+                p = ttime.plot.contour(
+                    ax=ax,
+                    transform=ccrs.PlateCarree(),
+                    linestyles="solid",
+                    add_labels=False,
+                    zorder=2*(1+ic),
+                    **c_args,
+                )
+            if timesize == 1:
+                ax.clabel(p, c3day["levels"], colors="black", fmt="%.0fdays")
 
             ax.set_global()
             ax.coastlines()
             ax.stock_img()
-            ax.plot(float(ds.lon0), float(ds.lat0), "or", transform=ccrs.PlateCarree())
+            ax.plot(lon0, lat0, "or", transform=ccrs.PlateCarree())
 
+        fig.suptitle(f"[{ds.start_time[:10]} - {ds.end_time[:10]}]")
         figs.append(fig)
         if outdir is not None:
             fig.savefig(os.path.join(outdir, f"estela_{grouper}.png"))
@@ -275,6 +276,7 @@ def dist_and_bearing(lat1, lat2, lon1, lon2):
     a = np.sin(latdifr / 2) * np.sin(latdifr / 2) + np.cos(lat1r) * np.cos(
         lat2r
     ) * np.sin(londifr / 2) * np.sin(londifr / 2)
+    a = a.clip(0., 1.) # to avoid warning for a=1.0000001,
     degdist = 2 * np.arctan2(np.sqrt(a), np.sqrt(1 - a)) / d2r
 
     y = np.sin(londifr) * np.cos(lat2r)
@@ -307,6 +309,7 @@ def geographic_mask(lat0, lon0, dists, bearings, mask=None):
     dmax = 180 * np.ones(360)
     if mask is None:
         mask = xr.ones_like(dists)
+    mask = mask.where(dists > 0, np.nan)
 
     valid = (mask != 1).values.flatten()
     d_arr = dists.values.flatten()[valid]
@@ -328,22 +331,9 @@ def geographic_mask(lat0, lon0, dists, bearings, mask=None):
 
 def get_groupers(groupers):
     if groupers is None:
-        groupers = ["ALL", "time.season"]
+        groupers = ["ALL", "time.season", "time.month"]
     return groupers
 
 
 if __name__ == "__main__":
     parser()
-    # mounted_g05 = "/wave/global/era5_glob-st4_prod/ww3*01_00z/glob201[89]??01T00.nc"
-    # local_g05 = "/data_local/tmp/glob2018??01T00.nc"
-    # local_g04 = "/data_local/tmp/ww3.glob_24m.2010??.nc"
-
-    # lat0 = 46  # -38  #, -13.76
-    # lon0 = -131  # 174.5  #, -172.07
-    # groupers = ["ALL"]  # ["ALL", "time.season", "time.month"]
-    # proj = None  # ccrs.Orthographic(lon0, lat0) # None
-
-    # # estelas = calc(local_g05, lat0, lon0, si=20, mask="MAPSTA", groupers=groupers)
-    # estelas = calc(local_g04, lat0, lon0, "hs.", "tp.", "th.", 20, "MAPSTA", groupers) # dsrp="si.""
-
-    # # TODO: plot(estelas, proj, groupers)
